package com.eden.common.core.controller;

import java.beans.PropertyEditorSupport;
import java.util.Date;
import java.util.List;

import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import javax.servlet.http.HttpServletRequest;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.eden.room.common.Result;
import com.eden.common.utils.DateUtils;
import com.eden.common.core.page.TableDataInfo;

/**
 * web层通用数据处理
 */
public class BaseController {
    /**
     * 将前台传递过来的日期格式的字符串，自动转化为Date类型
     */
    @InitBinder
    public void initBinder(WebDataBinder binder) {
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
            @Override
            public void setAsText(String text) {
                setValue(DateUtils.parseDate(text));
            }
        });
    }

    /**
     * 设置请求分页数据
     */
    protected void startPage() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        Integer pageNum = 1;
        Integer pageSize = 10;
        try {
            String pageNumStr = request.getParameter("pageNum");
            String pageSizeStr = request.getParameter("pageSize");
            if (pageNumStr != null && !pageNumStr.isEmpty()) {
                pageNum = Integer.parseInt(pageNumStr);
            }
            if (pageSizeStr != null && !pageSizeStr.isEmpty()) {
                pageSize = Integer.parseInt(pageSizeStr);
            }
        } catch (Exception e) {
            // 忽略转换异常，使用默认值
        }
        PageHelper.startPage(pageNum, pageSize);
    }

    /**
     * 设置请求分页数据
     */
    protected void startPage(Integer pageNum, Integer pageSize) {
        pageNum = (pageNum == null || pageNum < 1) ? 1 : pageNum;
        pageSize = (pageSize == null || pageSize < 1) ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize);
    }

    /**
     * 设置请求排序数据
     */
    protected void startOrderBy(String orderBy) {
        if (orderBy != null && !orderBy.isEmpty()) {
            PageHelper.orderBy(orderBy);
        }
    }

    /**
     * 响应请求分页数据
     */
    protected <T> PageInfo<T> getPageInfo(List<T> list) {
        return new PageInfo<>(list);
    }

    /**
     * 响应请求分页数据
     */
    protected <T> Result<PageInfo<T>> getPageResult(List<T> list) {
        PageInfo<T> pageInfo = new PageInfo<>(list);
        return Result.success(pageInfo);
    }

    /**
     * 响应返回结果
     */
    protected <T> Result<T> toResult(int rows) {
        return rows > 0 ? Result.success() : Result.error("操作失败");
    }

    /**
     * 响应返回结果
     */
    protected <T> Result<T> toResult(boolean result) {
        return result ? Result.success() : Result.error("操作失败");
    }

    /**
     * 响应返回结果
     */
    protected <T> Result<T> toResult(int rows, T data) {
        return rows > 0 ? Result.success(data) : Result.error("操作失败");
    }

    /**
     * 返回成功
     */
    public <T> Result<T> success() {
        return Result.success();
    }

    /**
     * 返回成功消息
     */
    public <T> Result<T> success(String message) {
        return Result.success(message, null);
    }

    /**
     * 返回成功数据
     */
    public <T> Result<T> success(T data) {
        return Result.success(data);
    }

    /**
     * 返回成功消息和数据
     */
    public <T> Result<T> success(String message, T data) {
        return Result.success(message, data);
    }

    /**
     * 返回失败消息
     */
    public <T> Result<T> error() {
        return Result.error();
    }

    /**
     * 返回失败消息
     */
    public <T> Result<T> error(String message) {
        return Result.error(message);
    }

    /**
     * 返回错误码消息
     */
    public <T> Result<T> error(String code, String message) {
        return Result.error(code, message);
    }

    /**
     * 页面跳转
     */
    public String redirect(String url) {
        return String.format("redirect:%s", url);
    }
} 