package com.eden.room.controller;

import com.eden.room.common.Result;
import com.eden.room.domain.RoomExport;
import com.eden.room.domain.dto.RoomExportDTO;
import com.eden.room.domain.dto.RoomImportDTO;
import com.eden.room.service.IRoomService;
import com.eden.room.service.RoomTemplateService;
import com.eden.room.utils.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * 房源模板下载控制器
 */
@Api(tags = "房源导入导出管理", description = "提供房源模板下载、导入导出相关接口")
@RestController
@RequestMapping("/room/template")
public class RoomTemplateController {
    
    private static final Logger log = LoggerFactory.getLogger(RoomTemplateController.class);
    
    @Autowired
    private RoomTemplateService roomTemplateService;
    
    @Autowired
    private IRoomService roomService;

    /**
     * 下载房源导入模板
     *
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "下载旧版房源导入模板", notes = "下载旧版Excel导入模板")
    @GetMapping("/download")
    public void downloadTemplate(HttpServletResponse response) throws IOException {
        try {
            // 设置要下载的文件的名称
            String fileName = "room_import.xlsx";
            
            // 设置响应头
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                    "attachment; filename=" + encodedFileName + ";filename*=utf-8''" + encodedFileName);
            
            // 获取模板文件资源
            ClassPathResource resource = new ClassPathResource("templates/excel/" + fileName);
            
            // 将文件内容写入响应流
            try (InputStream inputStream = resource.getInputStream();
                 OutputStream outputStream = response.getOutputStream()) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                outputStream.flush();
            }
        } catch (IOException e) {
            // 如果发生异常，写入错误响应
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            response.getWriter().write("{\"code\":\"1\",\"message\":\"下载模板文件失败: " + e.getMessage() + "\"}");
        }
    }


    /**
     * 批量导出房源信息
     * 
     * @param ids 房源ID数组
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "批量导出房源", notes = "根据房源ID数组批量导出房源详细信息为Excel文件")
    @ApiImplicitParam(name = "ids", value = "房源ID数组", required = true, dataType = "List")
    @PostMapping("/export/batch")
    public void exportRoomBatch(@RequestBody List<String> ids, HttpServletResponse response) {
        try {
            if (ids == null || ids.isEmpty()) {
                throw new IllegalArgumentException("房源ID不能为空");
            }

            // 将List<String>转换为String[]
            String[] idArray = ids.toArray(new String[0]);
            
            // 根据ID列表查询房源列表
            List<RoomExportDTO> list = roomService.exportRoomByIds(idArray);
            
            if (list.isEmpty()) {
                throw new IllegalArgumentException("未找到指定的房源信息");
            }
            
            // 设置要下载的文件名称
            String fileName = "房源信息_" + System.currentTimeMillis() + ".xlsx";
            
            // 转换为符合导出模板的数据结构
            List<RoomExport> exportData = roomTemplateService.convertToRoomImportList(list);
            
            // 设置响应头
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                    "attachment; filename=" + encodedFileName + ";filename*=utf-8''" + encodedFileName);
            
            // 导出Excel
            ExcelUtil<RoomExport> excelUtil = new ExcelUtil<>(RoomExport.class);
            excelUtil.exportExcel(exportData, "房源信息", response.getOutputStream());

        } catch (Exception e) {
            log.error("批量导出房源信息失败", e);
            // 如果发生异常，写入错误响应
            try {
                response.reset();
                response.setContentType("application/json");
                response.setCharacterEncoding("utf-8");
                response.getWriter().write("{\"code\":\"1\",\"message\":\"批量导出房源信息失败: " + e.getMessage() + "\"}");
            } catch (IOException ioException) {
                log.error("写入错误响应失败", ioException);
            }
        }
    }

    /**
     * 批量导入房源数据
     * 
     * @param file 导入房源的Excel文件（必须遵循指定的模板格式）
     * @param response HTTP响应对象
     */
    @ApiOperation(value = "批量导入房源", notes = "通过Excel文件批量导入房源数据，返回导入结果信息")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true, dataType = "MultipartFile")
    @PostMapping("/import")
    public void importRoom(@RequestParam("file") MultipartFile file, HttpServletResponse response) throws IOException {
        if (file.isEmpty()) {
            throw new RuntimeException("请选择要上传的文件");
        }
        
        // 校验文件格式
        String filename = file.getOriginalFilename();
        if (filename == null || !(filename.toLowerCase().endsWith(".xls") || filename.toLowerCase().endsWith(".xlsx"))) {
            throw new RuntimeException("文件格式不正确，请上传Excel文件");
        }
        
        log.info("开始导入Excel文件: {}, 大小: {} bytes", filename, file.getSize());
        
        try {
            // 检查文件大小（10MB限制）
            if (file.getSize() > 10 * 1024 * 1024) {
                throw new RuntimeException("文件过大，请控制在10MB以内");
            }
            
            Map<String, Object> result = roomService.importRoom(file);
            
            // 获取Excel数据流
            byte[] excelBytes = (byte[]) result.get("excelBytes");
            if (excelBytes != null && excelBytes.length > 0) {
                // 设置响应头
                response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
                response.setCharacterEncoding("utf-8");
                String encodedFileName = URLEncoder.encode("room_import_result.xlsx", StandardCharsets.UTF_8.toString());
                response.setHeader(HttpHeaders.CONTENT_DISPOSITION,
                        "attachment; filename=" + encodedFileName + ";filename*=utf-8''" + encodedFileName);
                
                // 将数据流写入响应
                try (OutputStream outputStream = response.getOutputStream()) {
                    outputStream.write(excelBytes);
                    outputStream.flush();
                }
            } else {
                throw new RuntimeException("生成结果文件失败");
            }
        } catch (Exception e) {
            log.error("导入房源失败", e);
            throw new RuntimeException("导入失败：" + e.getMessage());
        }
    }
} 