package com.eden.room.service.impl;


import com.eden.room.common.ContextUser;
import com.eden.room.common.UserContextHolder;
import com.eden.room.common.enums.ImageTypeEnum;
import com.eden.room.common.enums.PriceUnitEnum;
import com.eden.room.common.enums.RentalStatusEnum;
import com.eden.room.domain.*;
import com.eden.room.domain.dto.*;
import com.eden.room.feign.ContractFeignClient;
import com.eden.room.feign.CustomerFeignClient;
import com.eden.room.feign.TagFeignClient;
import com.eden.room.mapper.BuildingMapper;
import com.eden.room.mapper.FloorMapper;
import com.eden.room.mapper.ProjectMapper;
import com.eden.room.mapper.RoomBusinessInfoMapper;
import com.eden.room.mapper.RoomExtendInfoMapper;
import com.eden.room.mapper.RoomImageMapper;
import com.eden.room.mapper.RoomMapper;
import com.eden.room.service.IRoomService;
import com.eden.room.utils.ExcelUtil;
import com.eden.room.utils.excel.Excel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 房源服务实现类
 */
@Service
@Slf4j
public class RoomServiceImpl implements IRoomService {

    @Autowired
    private RoomMapper roomMapper;
    
    @Autowired
    private RoomBusinessInfoMapper businessInfoMapper;
    
    @Autowired
    private RoomExtendInfoMapper extendInfoMapper;
    
    @Autowired
    private RoomImageMapper imageMapper;
    
    @Autowired
    private ProjectMapper projectMapper;
    
    @Autowired
    private BuildingMapper buildingMapper;
    
    @Autowired
    private FloorMapper floorMapper;
    
    @Autowired
    private TagFeignClient tagFeignClient;
    
    @Autowired
    private CustomerFeignClient customerFeignClient;
    
    @Autowired
    private ContractFeignClient contractFeignClient;

    /**
     * 查询房源列表
     */
    @Override
    public List<Room> selectRoomList(Room room) {
        // 设置租户ID
        //setTenantId(room);
        return roomMapper.selectRoomList(room);
    }

    /**
     * 根据ID查询房源
     */
    @Override
    public Room selectRoomById(Long id) {
        return roomMapper.selectRoomById(id);
    }

    /**
     * 查询房源详情（包括基础信息、招商信息、扩展信息）
     */
    @Override
    public RoomDTO selectRoomDetailById(Long id) {
        // 查询房源基础信息
        Room room = roomMapper.selectRoomById(id);
        if (room == null) {
            return null;
        }
        
        RoomDTO roomDTO = new RoomDTO();
        BeanUtils.copyProperties(room, roomDTO);
        
        // 查询招商信息
        RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(id);
        roomDTO.setBusinessInfo(businessInfo);
        
        // 查询拓展信息
        RoomExtendInfo extendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(id);
        roomDTO.setExtendInfo(extendInfo);
        
        // 查询房源图片
        List<RoomImage> images = imageMapper.selectRoomImagesByRoomId(id);
        if (images != null && !images.isEmpty()) {
            // 过滤房源图片
            List<RoomImage> roomImages = images.stream()
                    .filter(img -> ImageTypeEnum.ROOM_IMAGE.getValue().equals(img.getImageType()))
                    .collect(Collectors.toList());
            roomDTO.setRoomImages(roomImages);
            
            // 过滤户型图
            List<RoomImage> floorPlanImages = images.stream()
                    .filter(img -> ImageTypeEnum.FLOOR_PLAN.getValue().equals(img.getImageType()))
                    .collect(Collectors.toList());
            roomDTO.setFloorPlanImages(floorPlanImages);
        }
        
        // 查询项目名称
        if (StringUtils.isNotEmpty(room.getProjectId())) {
            roomDTO.setProjectName(projectMapper.selectProjectById(room.getProjectId()).getProjectName());
        }
        
        // 查询楼宇名称
        if (room.getBuildingId()!=null) {
            roomDTO.setBuildingName(buildingMapper.selectBuildingById(room.getBuildingId()).getBuildingName());
        }
        
        // 查询楼层名称
        if (room.getFloorId()!=null) {
            roomDTO.setFloorName(floorMapper.selectFloorById(room.getFloorId()).getFloorName());
        }
        
        // 查询业主名称
        if (room.getOwnerId()!=null) {
            OwnerDTO owner = customerFeignClient.getOwnerById(room.getOwnerId());
            if (owner != null) {
                roomDTO.setOwnerName(owner.getName());
            }
        }
        
        // 查询标签
        if (extendInfo != null && StringUtils.isNotEmpty(extendInfo.getTags())) {
            String[] tagIds = extendInfo.getTags().split(",");
            List<TagDTO> tags = tagFeignClient.getTagsByIds(tagIds);
            roomDTO.setTags(roomDTO.getTags());
        }
        
        return roomDTO;
    }

    /**
     * 新增房源信息
     */
    @Override
    @Transactional
    public int insertRoom(RoomDTO roomDTO) {
        // 获取当前用户
        ContextUser user = new ContextUser();
        user.setUsername("admin");

        Date now = new Date();
        
        // 设置房源基础信息
        Room room = new Room();
        BeanUtils.copyProperties(roomDTO, room);
        //setTenantId(room);
        room.setCreateTime(now);
        room.setLastModTime(now);
        room.setCreateUserId(user.getUserName());
        room.setLastModUserId(user.getUserName());
        room.setDelFlag("0");
        
        // 插入房源基础信息
        int rows = roomMapper.insertRoom(room);
        Long roomId = room.getId();
        
        // 设置招商信息
        RoomBusinessInfo businessInfo = roomDTO.getBusinessInfo();
        if (businessInfo != null) {
            businessInfo.setRoomId(roomId);
            //setTenantId(businessInfo);
            businessInfo.setCreateTime(now);
            businessInfo.setLastModTime(now);
            businessInfo.setCreateUserId(user.getUserName());
            businessInfo.setLastModUserId(user.getUserName());
            businessInfo.setDelFlag("0");
            
            // 默认设置为待租状态
            if (StringUtils.isEmpty(businessInfo.getRentalStatus())) {
                businessInfo.setRentalStatus(RentalStatusEnum.WAITING.getKey());
            }
            businessInfo.setPriceUnit(PriceUnitEnum.getByValue(businessInfo.getPriceUnit()).getKey());
            // 插入招商信息
            businessInfoMapper.insertRoomBusinessInfo(businessInfo);
        }
        
        // 设置拓展信息
        RoomExtendInfo extendInfo = roomDTO.getExtendInfo();
        if (extendInfo != null) {
            extendInfo.setRoomId(roomId);
            //setTenantId(extendInfo);
            extendInfo.setCreateTime(now);
            extendInfo.setLastModTime(now);
            extendInfo.setCreateUserId(user.getUserName());
            extendInfo.setLastModUserId(user.getUserName());
            extendInfo.setDelFlag("0");
            
            // 处理标签
            if (roomDTO.getTags() != null && !roomDTO.getTags().isEmpty()) {
                extendInfo.setTags(roomDTO.getTags());
            }
            
            // 插入拓展信息
            extendInfoMapper.insertRoomExtendInfo(extendInfo);
        }
        
        // 插入房源图片
        insertRoomImages(roomDTO.getRoomImages(), roomId, ImageTypeEnum.ROOM_IMAGE.getValue(), user);
        
        // 插入户型图
        insertRoomImages(roomDTO.getFloorPlanImages(), roomId, ImageTypeEnum.FLOOR_PLAN.getValue(), user);
        
        return rows;
    }

    /**
     * 修改房源信息
     */
    @Override
    @Transactional
    public int updateRoom(RoomDTO roomDTO) {
        // 获取当前用户
        ContextUser user = new ContextUser();
        user.setUsername("admin");
        Date now = new Date();
        Long roomId = roomDTO.getId();
        
        // 设置房源基础信息
        Room room = new Room();
        BeanUtils.copyProperties(roomDTO, room);
        room.setLastModTime(now);
        room.setLastModUserId(user.getUserName());
        
        // 更新房源基础信息
        int rows = roomMapper.updateRoom(room);
        
        // 更新招商信息
        RoomBusinessInfo businessInfo = roomDTO.getBusinessInfo();
        if (businessInfo != null) {
            businessInfo.setRoomId(roomId);
            businessInfo.setLastModTime(now);
            businessInfo.setLastModUserId(user.getUserName());
            
            // 判断招商信息是否存在
            RoomBusinessInfo existBusinessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(roomId);
            if (existBusinessInfo != null) {
                businessInfoMapper.updateRoomBusinessInfo(businessInfo);
            } else {
                //setTenantId(businessInfo);
                businessInfo.setCreateTime(now);
                businessInfo.setCreateUserId(user.getUserName());
                businessInfo.setDelFlag("0");
                businessInfoMapper.insertRoomBusinessInfo(businessInfo);
            }
        }
        
        // 更新拓展信息
        RoomExtendInfo extendInfo = roomDTO.getExtendInfo();
        if (extendInfo != null) {
            extendInfo.setRoomId(roomId);
            extendInfo.setLastModTime(now);
            extendInfo.setLastModUserId(user.getUserName());
            
            // 处理标签
            if (roomDTO.getTags() != null) {
                extendInfo.setTags(roomDTO.getTags());
            }
            
            // 判断拓展信息是否存在
            RoomExtendInfo existExtendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(roomId);
            if (existExtendInfo != null) {
                extendInfoMapper.updateRoomExtendInfo(extendInfo);
            } else {
                //setTenantId(extendInfo);
                extendInfo.setCreateTime(now);
                extendInfo.setCreateUserId(user.getUserName());
                extendInfo.setDelFlag("0");
                extendInfoMapper.insertRoomExtendInfo(extendInfo);
            }
        }
        
        // 更新房源图片：先删除再新增
        if (roomDTO.getRoomImages() != null || roomDTO.getFloorPlanImages() != null) {
            // 删除原有图片
            imageMapper.deleteRoomImagesByRoomId(roomId);
            
            // 重新插入房源图片
            insertRoomImages(roomDTO.getRoomImages(), roomId, ImageTypeEnum.ROOM_IMAGE.getValue(), user);
            
            // 重新插入户型图
            insertRoomImages(roomDTO.getFloorPlanImages(), roomId, ImageTypeEnum.FLOOR_PLAN.getValue(), user);
        }
        
        return rows;
    }

    /**
     * 删除房源信息
     */
    @Override
    @Transactional
    public int deleteRoomById(Long id) {
        // 检查房源是否有正在执行的合同
        if (checkRoomContract(id)) {
            throw new RuntimeException("该房源有正在执行的合同，不能删除");
        }
        
        // 删除房源基础信息
        int rows = roomMapper.deleteRoomById(id);
        
        // 删除招商信息
        businessInfoMapper.deleteRoomBusinessInfoByRoomId(id);

        // 删除拓展信息
        extendInfoMapper.deleteRoomExtendInfoByRoomId(id);
        
        // 删除图片信息
        imageMapper.deleteRoomImagesByRoomId(id);
        
        return rows;
    }

    /**
     * 批量删除房源信息
     */
    @Override
    @Transactional
    public int deleteRoomByIds(String[] ids) {
        // 检查房源是否有正在执行的合同
        for (String id : ids) {
            if (checkRoomContract(Long.valueOf(id))) {
                throw new RuntimeException("房源ID为" + id + "的房源有正在执行的合同，不能删除");
            }
        }
        
        // 批量删除房源基础信息
        int rows = roomMapper.deleteRoomByIds(ids);
        
        // 批量删除招商信息
        businessInfoMapper.deleteRoomBusinessInfoByRoomIds(ids);
        
        // 批量删除拓展信息
        extendInfoMapper.deleteRoomExtendInfoByRoomIds(ids);
        
        // 批量删除图片信息
        imageMapper.deleteRoomImagesByRoomIds(ids);
        
        return rows;
    }

    /**
     * 导出房源信息
     */
    @Override
    public List<RoomExportDTO> exportRoom(Room room) {
        // 设置租户ID
        //setTenantId(room);
        
        // 查询房源信息
        List<Room> roomList = roomMapper.selectRoomList(room);
        if (roomList == null || roomList.isEmpty()) {
            return new ArrayList<>();
        }
        
        // 转换为导出DTO
        List<RoomExportDTO> exportList = new ArrayList<>();
        for (Room r : roomList) {
            RoomExportDTO exportDTO = new RoomExportDTO();
            
            // 设置项目名称
            if (StringUtils.isNotEmpty(r.getProjectId())) {
                exportDTO.setProjectName(projectMapper.selectProjectById(r.getProjectId()).getProjectName());
            }
            
            // 设置楼宇名称
            if (r.getBuildingId()!=null) {
                exportDTO.setBuildingName(buildingMapper.selectBuildingById(r.getBuildingId()).getBuildingName());
            }
            
            // 设置楼层名称
            if (r.getFloorId()!=null) {
                exportDTO.setFloorName(floorMapper.selectFloorById(r.getFloorId()).getFloorName());
            }
            
            // 设置基础信息
            exportDTO.setRoomNumber(r.getRoomNumber());
            exportDTO.setRoomDigitalNumber(r.getRoomDigitalNumber());
            exportDTO.setRentalArea(r.getRentalArea());
            exportDTO.setBillingArea(r.getBillingArea());
            exportDTO.setBuildingArea(r.getBuildingArea());
            exportDTO.setInnerArea(r.getInnerArea());
            exportDTO.setRoomManageNumber(r.getRoomManageNumber());

            // 查询招商信息
            RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(r.getId());
            if (businessInfo != null) {
                exportDTO.setPrice(businessInfo.getPrice());
                exportDTO.setPriceUnit(businessInfo.getPriceUnit());
                exportDTO.setRentalStatus(businessInfo.getRentalStatus());
                exportDTO.setAvailableDate(businessInfo.getAvailableDate());
                exportDTO.setBusinessStatus(businessInfo.getBusinessStatus());
                exportDTO.setDecorationStatus(businessInfo.getDecorationStatus());
            }
            
            // 查询拓展信息
            RoomExtendInfo extendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(r.getId());
            if (extendInfo != null) {
                exportDTO.setTags(extendInfo.getTags());
            }
            
            exportList.add(exportDTO);
        }
        
        return exportList;
    }

    /**
     * 修改房源租赁状态
     */
    @Override
    public int updateRoomRentalStatus(Long id, String rentalStatus) {
        // 校验租赁状态是否有效
        if (!RentalStatusEnum.isValid(rentalStatus)) {
            throw new RuntimeException("无效的租赁状态");
        }
        
        // 查询招商信息
        RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(id);
        if (businessInfo == null) {
            throw new RuntimeException("房源招商信息不存在");
        }
        
        // 设置租赁状态
        businessInfo.setRentalStatus(rentalStatus);
        businessInfo.setLastModTime(new Date());
        businessInfo.setLastModUserId(UserContextHolder.get().getUserName());
        
        return businessInfoMapper.updateRoomBusinessInfo(businessInfo);
    }

    /**
     * 检查房源是否有正在执行的合同
     */
    @Override
    public boolean checkRoomContract(Long id) {
        // 调用合同服务接口，检查房源是否有正在执行的合同
        return contractFeignClient.checkRoomContract(id, null);
    }
    
    /**
     * 根据ID列表导出房源信息
     *
     * @param ids 房源ID数组
     * @return 房源信息列表
     */
    @Override
    public List<RoomExportDTO> exportRoomByIds(String[] ids) {
        if (ids == null || ids.length == 0) {
            return new ArrayList<>();
        }
        
        List<RoomExportDTO> exportList = new ArrayList<>();
        
        // 查询指定ID的房源信息
        List<Room> roomList = roomMapper.selectRoomByIds(ids);
        if (roomList.isEmpty()) {
            return exportList;
        }
        
        // 转换为导出DTO
        for (Room r : roomList) {
            RoomExportDTO exportDTO = new RoomExportDTO();
            
            // 复制基本信息
            exportDTO.setId(r.getId());
            exportDTO.setProjectId(r.getProjectId());
            exportDTO.setProjectName(r.getProjectName());
            exportDTO.setBuildingId(r.getBuildingId());
            exportDTO.setBuildingName(r.getBuildingName());
            exportDTO.setFloorId(r.getFloorId());
            exportDTO.setFloorName(r.getFloorName());
            exportDTO.setRoomNumber(r.getRoomNumber());
            exportDTO.setRoomDigitalNumber(r.getRoomDigitalNumber());
            exportDTO.setRoomManageNumber(r.getRoomManageNumber());
            exportDTO.setRoomName(r.getRoomName());
            exportDTO.setRoomType(r.getRoomType());
            exportDTO.setFloor(r.getFloor());
            exportDTO.setIsWholeFloor(r.getIsWholeFloor());
            exportDTO.setBuildingArea(r.getBuildingArea());
            exportDTO.setRentalArea(r.getRentalArea());
            exportDTO.setBillingArea(r.getBillingArea());
            exportDTO.setInnerArea(r.getInnerArea());
            exportDTO.setIsVirtual(r.getIsVirtual());
            exportDTO.setDeliveryTime(r.getDeliveryTime());
            exportDTO.setOwnerId(r.getOwnerId());
            exportDTO.setPropertyNature(r.getPropertyNature());
            exportDTO.setHeight(r.getHeight());
            exportDTO.setLoadValue(r.getLoadValue());
            exportDTO.setRoomStatus(r.getRoomStatus());
            exportDTO.setCreateTime(r.getCreateTime());

            
            // 查询招商信息
            RoomBusinessInfo businessInfo = businessInfoMapper.selectRoomBusinessInfoByRoomId(r.getId());
            if (businessInfo != null) {
                exportDTO.setPrice(businessInfo.getPrice());
                exportDTO.setPriceUnit(businessInfo.getPriceUnit());
                exportDTO.setRentalStatus(businessInfo.getRentalStatus());
                exportDTO.setAvailableDate(businessInfo.getAvailableDate());
                exportDTO.setBusinessStatus(businessInfo.getBusinessStatus());
                exportDTO.setDecorationStatus(businessInfo.getDecorationStatus());
            }
            
            // 查询拓展信息
            RoomExtendInfo extendInfo = extendInfoMapper.selectRoomExtendInfoByRoomId(r.getId());
            if (extendInfo != null) {
                exportDTO.setTags(extendInfo.getTags());
            }
            
            exportList.add(exportDTO);
        }
        
        return exportList;
    }
    
    /**
     * 插入房源图片
     *
     * @param images 图片列表
     * @param roomId 房源ID
     * @param imageType 图片类型
     * @param user 当前用户
     */
    private void insertRoomImages(List<RoomImage> images, Long roomId, String imageType, ContextUser user) {
        if (images != null && !images.isEmpty()) {
            Date now = new Date();
            int sortOrder = 0;
            
            for (RoomImage image : images) {
                image.setRoomId(roomId);
                image.setImageType(imageType);
                image.setSortOrder(sortOrder++);
                //setTenantId(image);
                image.setCreateTime(now);
                image.setLastModTime(now);
                image.setCreateUserId(user.getUserName());
                image.setLastModUserId(user.getUserName());
                image.setDelFlag("0");
                
                imageMapper.insertRoomImage(image);
            }
        }
    }
    
    /**
     * 设置租户ID
     *
     * @param obj 需要设置租户ID的对象
     */
//    private void setTenantId(Object obj) {
//        if (obj instanceof Room) {
//            ((Room) obj).setTenantId(UserContextHolder.get().getTenantId());
//        } else if (obj instanceof RoomBusinessInfo) {
//            ((RoomBusinessInfo) obj).setTenantId(UserContextHolder.get().getTenantId());
//        } else if (obj instanceof RoomExtendInfo) {
//            ((RoomExtendInfo) obj).setTenantId(UserContextHolder.get().getTenantId());
//        } else if (obj instanceof RoomImage) {
//            ((RoomImage) obj).setTenantId(UserContextHolder.get().getTenantId());
//        }
//    }

    /**
     * 批量导入房源信息
     *
     * @param file 导入文件
     * @return 导入结果信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map<String, Object> importRoom(MultipartFile file) throws Exception {
        // 初始化结果
        Map<String, Object> result = new HashMap<>();
        int successCount = 0;
        int failCount = 0;
        StringBuilder failMsg = new StringBuilder();

        // 解析Excel文件
        ExcelUtil<RoomImportDTO> excelUtil = new ExcelUtil<>(RoomImportDTO.class);
        List<RoomImportDTO> roomImportList = excelUtil.importExcel(file.getInputStream());

        if (roomImportList.isEmpty()) {
            throw new RuntimeException("Excel文件中没有数据");
        }

        // 当前用户
        ContextUser user = UserContextHolder.get();
        Date now = new Date();

        // 处理每一行数据
        for (int i = 0; i < roomImportList.size(); i++) {
            RoomImportDTO roomImport = roomImportList.get(i);
            try {
                // 1. 查找项目
                String projectId = getProjectIdByName(roomImport.getProjectName());
                if (projectId == null) {
                    failCount++;
                    String errorMsg = "项目[" + roomImport.getProjectName() + "]不存在";
                    failMsg.append("第").append(i + 2).append("行：").append(errorMsg).append("<br/>");
                    roomImport.setImportResult(errorMsg);
                    continue;
                }

                // 2. 查找楼宇
                Long buildingId = getBuildingIdByName(projectId, roomImport.getBuildingName());
                if (buildingId == null) {
                    failCount++;
                    String errorMsg = "楼宇[" + roomImport.getBuildingName() + "]不存在";
                    failMsg.append("第").append(i + 2).append("行：").append(errorMsg).append("<br/>");
                    roomImport.setImportResult(errorMsg);
                    continue;
                }

                // 3. 查找楼层
                Long floorId = getFloorIdByName(buildingId, roomImport.getFloorName());
                if (floorId == null) {
                    failCount++;
                    String errorMsg = "楼层[" + roomImport.getFloorName() + "]不存在";
                    failMsg.append("第").append(i + 2).append("行：").append(errorMsg).append("<br/>");
                    roomImport.setImportResult(errorMsg);
                    continue;
                }

                // 4. 检查房源编号是否重复
                boolean roomExists = checkRoomCodeExists(roomImport.getRoomNumber(), projectId, buildingId, floorId);
                if (roomExists) {
                    failCount++;
                    String errorMsg = "房源编号[" + roomImport.getRoomNumber() + "]已存在";
                    failMsg.append("第").append(i + 2).append("行：").append(errorMsg).append("<br/>");
                    roomImport.setImportResult(errorMsg);
                    continue;
                }

                // 5. 保存房源基本信息
                Room room = new Room();
                room.setProjectId(projectId);
                room.setProjectName(roomImport.getProjectName());
                room.setBuildingId(buildingId);
                room.setBuildingName(roomImport.getBuildingName());
                room.setFloorId(floorId);
                room.setFloorName(roomImport.getFloorName());
                room.setRoomNumber(roomImport.getRoomNumber());
                room.setRoomDigitalNumber(roomImport.getRoomDigitalNumber());
                room.setRoomManageNumber(roomImport.getRoomManageNumber());
                room.setBuildingArea(roomImport.getBuildingArea());
                room.setRentalArea(roomImport.getRentalArea());
                room.setBillingArea(roomImport.getBillingArea());
                room.setHeight(roomImport.getHeight());
                room.setIsVirtual("0"); // 默认非虚拟房源
                room.setRoomStatus("1"); // 默认公开状态
                room.setDelFlag("0");
                room.setCreateTime(now);
                room.setLastModTime(now);

                // 插入房源基本信息
                roomMapper.insertRoom(room);

                // 6. 创建招商信息
                RoomBusinessInfo businessInfo = new RoomBusinessInfo();
                businessInfo.setRoomId(room.getId());
                businessInfo.setPrice(roomImport.getPrice());
                businessInfo.setFloorPrice(roomImport.getFloorPrice());
                businessInfo.setPriceUnit(roomImport.getPriceUnit());
                businessInfo.setRentalStatus("1"); // 默认待租状态
                businessInfo.setBusinessStatus("1"); // 默认招商状态
                businessInfo.setDecorationStatus("3"); // 默认毛坯状态
                businessInfo.setDelFlag("0");
                businessInfo.setCreateTime(now);
                businessInfo.setLastModTime(now);

                // 插入招商信息
                businessInfoMapper.insertRoomBusinessInfo(businessInfo);

                // 7. 创建拓展信息
                if (StringUtils.isNotBlank(roomImport.getTags())) {
                    RoomExtendInfo extendInfo = new RoomExtendInfo();
                    extendInfo.setRoomId(room.getId());
                    extendInfo.setTags(roomImport.getTags());
                    extendInfo.setDelFlag("0");
                    extendInfo.setCreateTime(now);
                    extendInfo.setLastModTime(now);

                    // 插入拓展信息
                    extendInfoMapper.insertRoomExtendInfo(extendInfo);
                }

                successCount++;
                roomImport.setImportResult("导入成功");
            } catch (Exception e) {
                failCount++;
                String errorMsg = "导入失败：" + e.getMessage();
                failMsg.append("第").append(i + 2).append("行：").append(errorMsg).append("<br/>");
                roomImport.setImportResult(errorMsg);
            }
        }

        // 只有在有导入失败的情况下才生成Excel文件
        byte[] excelBytes = null;
        if (failCount > 0) {
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            exportToExcelWithTemplate(roomImportList,  outputStream);
            excelBytes = outputStream.toByteArray();
            result.put("excelBytes", excelBytes);
        }
        result.put("successCount", successCount);
        result.put("failCount", failCount);
        result.put("failMsg", failMsg.toString());
        result.put("allSuccess", failCount == 0);

        return result;
    }
    /**
     * 使用模板文件导出Excel
     *
     * @param roomList 房源数据列表
     * @param outputStream 输出流
     * @throws IOException IO异常
     */
    public void exportToExcelWithTemplate(List<RoomImportDTO> roomList, OutputStream outputStream) throws IOException {
        // 加载模板文件
        ClassPathResource resource = new ClassPathResource("templates/room_import.xlsx");
        InputStream inputStream = resource.getInputStream();
        Workbook workbook = WorkbookFactory.create(inputStream);
        inputStream.close();

        // 获取第一个工作表
        Sheet sheet = workbook.getSheetAt(0);

        // 获取标题行
        Row headerRow = sheet.getRow(0);
        if (headerRow == null) {
            throw new RuntimeException("模板文件格式错误：未找到标题行");
        }

        // 获取字段映射
        List<Field> fields = getExcelFields(RoomExport.class);
        Map<Integer, Field> columnMap = new HashMap<>();
        for (int i = 0; i < headerRow.getLastCellNum(); i++) {
            Cell cell = headerRow.getCell(i);
            if (cell != null) {
                String headerName = cell.getStringCellValue();
                for (Field field : fields) {
                    Excel excel = field.getAnnotation(Excel.class);
                    if (excel != null && excel.name().equals(headerName)) {
                        columnMap.put(i, field);
                        break;
                    }
                }
            }
        }

        // 填充数据行
        int startRow = 4; // 从第4行开始填充数据
        for (RoomImportDTO room : roomList) {
            Row row = sheet.getRow(startRow);
            if (row == null) {
                row = sheet.createRow(startRow);
            }

            for (Map.Entry<Integer, Field> entry : columnMap.entrySet()) {
                int columnIndex = entry.getKey();
                Field field = entry.getValue();

                Cell cell = row.getCell(columnIndex);
                if (cell == null) {
                    cell = row.createCell(columnIndex);
                }

                field.setAccessible(true);
                try {
                    Object value = field.get(room);
                    if (value != null) {
                        cell.setCellValue(value.toString());
                    }
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }

            startRow++;
        }

        // 写入输出流
        workbook.write(outputStream);
        workbook.close();
    }
    private List<Field> getExcelFields(Class<?> clazz) {
        List<Field> fields = new ArrayList<>();
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            if (field.isAnnotationPresent(Excel.class)) {
                fields.add(field);
            }
        }
        // 按照sort属性排序
        fields.sort((f1, f2) -> {
            Excel excel1 = f1.getAnnotation(Excel.class);
            Excel excel2 = f2.getAnnotation(Excel.class);
            return excel1.sort() - excel2.sort();
        });
        return fields;
    }
    /**
     * 根据项目名称获取项目ID
     *
     * @param projectName 项目名称
     * @return 项目ID
     */
    private String getProjectIdByName(String projectName) {
        if (StringUtils.isBlank(projectName)) {
            return null;
        }
        Project param = new Project();
        param.setProjectName(projectName);
        return projectMapper.selectProjectIdByName(projectName);
    }
    
    /**
     * 根据楼宇名称获取楼宇ID
     *
     * @param projectId 项目ID
     * @param buildingName 楼宇名称
     * @return 楼宇ID
     */
    private Long getBuildingIdByName(String projectId, String buildingName) {
        if (StringUtils.isBlank(projectId) || StringUtils.isBlank(buildingName)) {
            return null;
        }
        
        Building param = new Building();
        param.setProjectId(projectId);
        param.setBuildingName(buildingName);
        List<Building> buildings = buildingMapper.selectBuildingList(param);
        return buildings.isEmpty() ? null : buildings.get(0).getId();
    }
    
    /**
     * 根据楼层名称获取楼层ID
     *
     * @param buildingId 楼宇ID
     * @param floorName 楼层名称
     * @return 楼层ID
     */
    private Long getFloorIdByName(Long buildingId, String floorName) {
        if (buildingId == null || StringUtils.isBlank(floorName)) {
            return null;
        }
        
        Floor param = new Floor();
        param.setBuildingId(buildingId);
        param.setFloorName(floorName);
        List<Floor> floors = floorMapper.selectFloorList(param);
        return floors.isEmpty() ? null : floors.get(0).getId();
    }
    
    /**
     * 检查房源编号是否已存在
     *
     * @param roomCode 房源编号
     * @param projectId 项目ID
     * @param buildingId 楼宇ID
     * @param floorId 楼层ID
     * @return 是否存在
     */
    private boolean checkRoomCodeExists(String roomCode, String projectId, Long buildingId, Long floorId) {
        if (StringUtils.isBlank(roomCode)) {
            return false;
        }
        
        Room param = new Room();
        param.setRoomNumber(roomCode);
        param.setProjectId(projectId);
        param.setBuildingId(buildingId);
        param.setFloorId(floorId);
        List<Room> rooms = roomMapper.selectRoomList(param);
        return !rooms.isEmpty();
    }
} 